EpisodicTopoART <- function() {
	
#--------------------------------------------------------------------------------------------------
	
	setClass("EpisodicTopoARTNode",
		representation(
			node_ID					=	"numeric",
			cluster_ID				=	"numeric",
			dim						=	"numeric",
			represented_inputs		=	"numeric",
			activation				=	"numeric",
			match_value				=	"numeric",
			edge_num				=	"numeric",
			edge_targets			=	"vector",
			weights					=	"vector",
			temporal_match_value	=	"numeric",
			combined_match_value	=	"numeric",
			temporal_weights		=	"vector"
		),
		prototype(
			node_ID					=	0,
			cluster_ID				=	0,
			dim						=	0,
			represented_inputs		=	0,
			activation				=	0,
			match_value				=	0,
			edge_num				=	0,
			edge_targets			=	vector(mode = "list", length = 0),
			weights					=	vector(mode = "list", length = 0),
			temporal_match_value	=	0,
			combined_match_value	=	0,
			temporal_weights		=	vector(mode = "list", length = 0)
		)
	)
	
	setMethod("initialize", "EpisodicTopoARTNode", function(.Object, src_file, node_index)
	{
		#tmpstring <- sprintf("load Episodic TopoART node %i", node_index - 1)
		#print(tmpstring)
		
		f <- src_file
		l <- readLines(f, n = 1)
		
		node_ID				<-	ReadNumeric(f, "node ID: %i");
		cluster_ID			<-	ReadNumeric(f, "cluster ID: %i");
		dim					<-	ReadNumeric(f, "input length: %i");
		represented_inputs	<-	ReadNumeric(f, "represented inputs: %i");
		activation			<-	ReadNumeric(f, "activation: %f");
		match_value			<-	ReadNumeric(f, "match value: %f");
				
		l <- readLines(f, n = 1)
		edge_num <- ReadNumeric(f, "edge number: %i");
		edge_targets <- vector(mode = "numeric", length = edge_num)
		l <- readLines(f, n = 1)
		substrings <- strsplit(l, split = " ")
		for(i in seq_len(edge_num))
			edge_targets[i] <- substrings[[1]][i + 2]
		#print(edge_num)
		#print(edge_targets)
		
		l <- readLines(f, n = 1)
		
		weights <- vector(mode = "numeric", length = dim)
		l <- readLines(f, n = 1)
		substrings <- strsplit(l, split = " ")
		for(i in seq_len(dim))
			weights[i] <- as.numeric(substrings[[1]][i])
		#print(weights)
		
		l <- readLines(f, n = 1)
		
		temporal_match_value	<-	ReadNumeric(f, "temporal_match_value: %f");
		combined_match_value	<-	ReadNumeric(f, "combined_match_value: %f");
		
		l <- readLines(f, n = 1)
		temporal_weights <- vector(mode = "numeric", length = 2)
		l <- readLines(f, n = 1)
		substrings <- strsplit(l, split = " ")
		temporal_weights[1] <- as.numeric(substrings[[1]][1])
		temporal_weights[2] <- as.numeric(substrings[[1]][2])
		#print(temporal_weights)
		
		.Object@node_ID					<-	node_ID
		.Object@cluster_ID				<-	cluster_ID
		.Object@dim						<-	dim
		.Object@represented_inputs		<-	represented_inputs
		.Object@activation				<-	activation
		.Object@match_value				<-	match_value
		.Object@edge_num				<-	edge_num
		.Object@edge_targets			<-	edge_targets
		.Object@weights					<-	weights
		.Object@temporal_match_value	<-	temporal_match_value
		.Object@combined_match_value	<-	combined_match_value
		.Object@temporal_weights		<-	temporal_weights
		.Object
	})
	
#--------------------------------------------------------------------------------------------------
	
	setClass("EpisodicTopoARTModule",
		representation(
			t_F1				=	"vector",
			x_F1_dim			=	"numeric",
			x_F1				=	"vector",
			rho					=	"numeric",
			next_ID				=	"numeric",
			valid_cluster_IDs	=	"logical",
			cluster_num			=	"numeric",
			LearningCycles		=	"numeric",
			node_num			=	"numeric",
			nodes				=	"vector"
		),
		prototype(
			t_F1				=	vector(mode = "numeric", length = 0),
			x_F1_dim			=	0,
			x_F1				=	vector(mode = "numeric", length = 0),
			rho					=	0,
			next_ID				=	0,
			valid_cluster_IDs	=	FALSE,
			cluster_num			=	0,
			LearningCycles		=	0,
			node_num			=	0,
			nodes				=	vector(mode = "list", length = 0)
		)
	)
	
	setMethod("initialize", "EpisodicTopoARTModule", function(.Object, src_file, module_index)
	{
		letters <- c("a", "b", "c", "d", "e")
		tmpstring <- sprintf("load  Episodic TopoART module %s", letters[module_index])
		print(tmpstring)
		
		f <- src_file
		l <- readLines(f, n = 3)
		
		t_F1 <- vector(mode = "numeric", length = 2)
		l <- readLines(f, n = 1)
		substrings <- strsplit(l, split = " ")
		num_string <- substrings[[1]][2]
		t_F1[1] <- as.numeric(num_string)
		num_string <- substrings[[1]][3]
		t_F1[2] <- as.numeric(num_string)
		#tmpstring <- sprintf("t_F1: %f %f", t_F1[1], t_F1[2])
		#print(tmpstring)
		
		x_F1_dim <- ReadNumeric(f, "x^F1 length: %i");
		
		x_F1 <- vector(mode = "numeric", length = x_F1_dim)
		l <- readLines(f, n = 1)
		substrings <- strsplit(l, split = " ")
		for(i in seq_len(x_F1_dim))
			x_F1[i] <- substrings[[1]][i + 1]
		#print("x_F1:")
		#print(x_F1)
		
		rho		<-	ReadNumeric(f, "rho: %f");
		next_ID	<-	ReadNumeric(f, "next ID: %i");
		
		l <- readLines(f, n = 1)
		substrings <- strsplit(l, split = " ")
		logical_string <- substrings[[1]][4]
		valid_cluster_IDs <- (logical_string == "true")
		if(valid_cluster_IDs) {
			tmpstring2 <- "TRUE"
		} else {
			tmpstring2 <- "FALSE"
		}
		#tmpstring <- sprintf("valid_cluster_IDs: %s", tmpstring2)
		#print(tmpstring)
		
		cluster_num		<-	ReadNumeric(f, "cluster number: %i");
		LearningCycles	<-	ReadNumeric(f, "learning cycles: %i");
		node_num		<-	ReadNumeric(f, "node number: %i");		
		
		nodes <- vector(mode = "list", length = 0)	
		for(i in seq_len(node_num))
			nodes[[i]] <- new("EpisodicTopoARTNode", f, i)
		
		.Object@t_F1				<-	t_F1
		.Object@x_F1_dim			<-	x_F1_dim
		.Object@x_F1				<-	x_F1
		.Object@rho					<-	rho
		.Object@next_ID				<-	next_ID
		.Object@valid_cluster_IDs	<-	valid_cluster_IDs
		.Object@cluster_num			<-	cluster_num
		.Object@LearningCycles		<-	LearningCycles
		.Object@node_num			<-	node_num
		.Object@nodes				<-	nodes
		.Object
	})
	
#--------------------------------------------------------------------------------------------------
	
	setClass("EpisodicTopoARTNetwork", 
		representation(
			LibTopoART_version		=	"numeric",
			file_format_version		=	"numeric",
			TA_file_format_version	=	"numeric",
			t_F0					=	"numeric",
			x_F0_dim				=	"numeric",
			x_F0					=	"vector",
			Rho_a					=	"numeric",
			Beta_sbm				=	"numeric",
			Tau						=	"numeric",
			Phi						=	"numeric",
			LearningSteps			=	"numeric",
			Alpha					=	"numeric",
			ModuleNum				=	"numeric",
			modules					=	"vector"
		), 
		prototype(
			LibTopoART_version		=	0,
			file_format_version		=	0,
			TA_file_format_version	=	0,
			t_F0					=	0,
			x_F0_dim				=	0,
			x_F0					=	vector(mode = "numeric", length = 0),
			Rho_a					=	0,
			Beta_sbm				=	0,
			Tau						=	0,
			Phi						=	0,
			LearningSteps			=	0,
			Alpha					=	0,
			ModuleNum				=	0,
			modules					=	vector(mode = "list", length = 0)			
		)
	)
	
	setMethod("initialize", "EpisodicTopoARTNetwork", function(.Object, path)
	{
		tmpstring <- sprintf("load Episodic TopoaART network: %s", path)
		print(tmpstring)
		
		f <- file(path, "r")
		l <- readLines(f, n = 1)
		l <- readLines(f, n = 1)
		if(!(l == "*   Episodic TopoART network    *")) {
			warning("No Episodic TopoART text file")
			close(f)
			return(.Object)
		}
		
		l <- readLines(f, n = 1)
		substrings <- strsplit(l, split = "v")
		version_string <- substr(substrings[[1]][2], 1, 4)
		LibTopoART_version <- as.numeric(version_string)
		tmpstring <- sprintf("LibTopoART v%.2f", LibTopoART_version)
		print(tmpstring)
		l <- readLines(f, n = 1)
		
		l <- readLines(f, n = 1)
		substrings <- strsplit(l, split = " ")
		version_string <- substr(substrings[[1]][4], 1, 4)
		file_format_version <- as.numeric(version_string)
		version_string <- substr(substrings[[1]][5], 1, 4)
		TA_file_format_version <- as.numeric(version_string)
		#tmpstring <- sprintf("file format versions: %.2f; %.2f", file_format_version, TA_file_format_version)
		#print(tmpstring)
		if((file_format_version != 0.02) || (TA_file_format_version != 0.10)) {
			warning("File format version not supported")
			close(f)
			return(.Object)
		}
		
		l <- readLines(f, n = 3)
		
		t_F0	<-	ReadNumeric(f, "t^F0: %i");	
		t_max	<-	ReadNumeric(f, "t^max: %i");	

		l <- readLines(f, n = 1)
		
		x_F0_dim <- ReadNumeric(f, "x^F0 length: %i")
		
		x_F0 <- vector(mode = "numeric", length = x_F0_dim)
		l <- readLines(f, n = 1)
		substrings <- strsplit(l, split = " ")
		for(i in seq_len(x_F0_dim))
			x_F0[i] <- substrings[[1]][i + 1]
		#print("x_F0:")
		#print(x_F0)
		
		Rho_a			<-	ReadNumeric(f, "rho_a: %f");
		Beta_sbm		<-	ReadNumeric(f, "beta_sbm: %f");		
		Tau				<-	ReadNumeric(f, "tau: %i");			
		Phi				<-	ReadNumeric(f, "phi: %i");	
		LearningSteps	<-	ReadNumeric(f, "learning steps: %i");		
		Alpha			<-	ReadNumeric(f, "alpha: %f");	
		ModuleNum		<-	ReadNumeric(f, "module number: %i");
		
		modules <- vector(mode = "list", length = 0)	
		for(i in seq_len(ModuleNum))
			modules[[i]] <- new("EpisodicTopoARTModule", f, i)
		
		close(f)
		
		.Object@LibTopoART_version		<-	LibTopoART_version
		.Object@file_format_version		<-	file_format_version
		.Object@TA_file_format_version	<-	TA_file_format_version
		.Object@t_F0					<-	t_F0
		.Object@x_F0_dim				<-	x_F0_dim
		.Object@x_F0					<-	x_F0
		.Object@Rho_a					<-	Rho_a
		.Object@Beta_sbm				<-	Beta_sbm
		.Object@Tau						<-	Tau
		.Object@Phi						<-	Phi
		.Object@LearningSteps			<-	LearningSteps
		.Object@Alpha					<-	Alpha
		.Object@ModuleNum				<-	ModuleNum
		.Object@modules					<-	modules
		.Object
	})
	
	setGeneric("SaveNetworkPlot", function(this) standardGeneric("SaveNetworkPlot"))
	setMethod("SaveNetworkPlot", "EpisodicTopoARTNetwork", function(this)
	{
		if(this@x_F0_dim != 2) {
            warning("Incompatible input length")
    	} else {
			ColorShift <- 4
			MaxClusterNum<-this@modules[[this@ModuleNum]]@cluster_num
			colors <- rainbow(MaxClusterNum)
			colors[1] <- "#ffff80FF"
			colors[2] <- "#00A000FF"
			letters <- c("a", "b", "c", "d", "e")
			
			for(m in seq_len(this@ModuleNum)) {
				tmpstring <- sprintf("images/ETA_%s.pdf", letters[m])
				pdf(file = tmpstring)
				palette(colors)
				par(cex.axis = 1.5, cex.main = 1.6)
				plot(NULL, NULL, xlim = c(0, 1), ylim = c(0, 1), xlab = "", ylab="")
				grid()
				tmpstring <- sprintf("ETA %s: rho_a=%.2f, beta_sbm=%.2f, phi=%i, tau=%i", letters[m], this@Rho_a, this@Beta_sbm, this@Phi, this@Tau)
				title(main = tmpstring, font.main = 2)

				for(n in seq_len(this@modules[[m]]@node_num)) {
					x1 <- this@modules[[m]]@nodes[[n]]@weights[[1]]
					y1 <- this@modules[[m]]@nodes[[n]]@weights[[2]]
					x2 <- 1 - this@modules[[m]]@nodes[[n]]@weights[[3]]
					y2 <- 1 - this@modules[[m]]@nodes[[n]]@weights[[4]]
					if(this@modules[[m]]@nodes[[n]]@cluster_ID > 0) {
						rect(x1, y1, x2, y2, col = (this@modules[[m]]@nodes[[n]]@cluster_ID - 1 + ColorShift) %% MaxClusterNum + 1)	
					}
				}

				dev.off()
			}
		}
	})	
	
#--------------------------------------------------------------------------------------------------

	setClass("EpisodicTopoARTRecallViewer",
		representation(
			path = "character"
		),
		prototype(
			path = ""
		)
	)
	
	setMethod("initialize", "EpisodicTopoARTRecallViewer", function(.Object, path)
	{
		.Object@path <- path
		.Object
	})
	
	setGeneric("SaveRecallPlot", function(this, max_inter_steps = -1, max_intra_steps = -1) standardGeneric("SaveRecallPlot"))
	setMethod("SaveRecallPlot", "EpisodicTopoARTRecallViewer", function(this, max_inter_steps, max_intra_steps)
	{
		
#--------------------------------------------------------------------------------------------------
# helper functions
#--------------------------------------------------------------------------------------------------
		
		CountImages <- function() 
		{
			filename		<-	sprintf("%sinter_image_%03i.jpg", this@path, inter_num + 1)
			
			while(file.exists(filename)) {
				#print(paste("count", filename))
				intra_num	<-	0
				filename	<-	sprintf("%sintra_image_%03i_%03i.jpg", this@path, inter_num + 1, intra_num + 1)
				
				while(file.exists(filename)) {
					#print(paste("count", filename))
					intra_num <- intra_num + 1
					filename <- sprintf("%sintra_image_%03i_%03i.jpg", this@path, inter_num + 1, intra_num + 1)
				}
				
				intra_num_array	<<-	c(intra_num_array, intra_num)
				max_intra_num	<<-	max(intra_num, max_intra_num)
				inter_num		<<-	inter_num + 1
				filename		<-	sprintf("%sinter_image_%03i.jpg", this@path, inter_num + 1)
			}
		}
		
		ShowInterEpisodeRecallResults <- function() 
		{
			if(plot_inter_num == 0)
				return()
		
			inter_ypos1 	<-	plot_yshift - (plot_inter_num + 1) * stimulus_dim[1] - plot_inter_num * image_dist - yoff - image_box - plot_pyoff
			inter_ypos2 	<-	plot_yshift - stimulus_dim[1] - image_dist - yoff + image_box
			inter_text_xpos	<-	xoff - image_box - larrow_off - recall_text_off	 / 2
			inter_text_ypos	<-	inter_ypos1 - (inter_ypos1 - inter_ypos2) / 2
			arrow_list		<-	list(NULL, NULL, NULL, NULL)
			
			if(inter_ypos2 - inter_ypos1 >= recall_text_min_size) {
				text(
					inter_text_xpos,
					inter_text_ypos,
					"inter-episode recall", adj = c(0.5, 0.5), cex = plot_cex, col = dark_red, srt = 90)
			}
			
			if(inter_ypos2 > inter_ypos1) {
				rect(
					xoff - image_box,
					inter_ypos1,
					stimulus_dim[2] + xoff + image_box,
					inter_ypos2,
					col = light_red,
					border = dark_red)
				
				inter_arrow_xpos <- xoff - image_box - larrow_off / 2
			
				arrows(
					inter_arrow_xpos,
					inter_ypos2,
					inter_arrow_xpos,
					inter_ypos1,
					col = dark_red,
					length = larrow_length_bounded)
			}
			
			if(plot_inter_num > 0) {
			
				# draw small arrows
				for(i in seq_len(ifelse(plot_inter_num < inter_num, plot_inter_num + 1, plot_inter_num))) {
					arrow_list[[1]] <- c(arrow_list[[1]], xoff + stimulus_dim[2] / 2)
					arrow_list[[2]] <- c(arrow_list[[2]], plot_yshift - i * stimulus_dim[1] - (i - 1) * image_dist - yoff)
					arrow_list[[3]] <- c(arrow_list[[3]], xoff + stimulus_dim[2] / 2)
					arrow_list[[4]] <- c(arrow_list[[4]], plot_yshift - i * stimulus_dim[1] - (i - 1) * image_dist - yoff - image_dist)
				}
				arrows(arrow_list[[1]], arrow_list[[2]], arrow_list[[3]], arrow_list[[4]], col = black, length = sarrow_length_bounded);
			
				# show ...
				if(plot_inter_num < inter_num) {
					text(
						xoff + stimulus_dim[2] / 2,
						plot_yshift - (plot_inter_num + 1) * (stimulus_dim[1] + image_dist) - yoff - points_yoff / 2,	
						points_string, adj = c(0.5, 0.5), cex = plot_cex, col = black)
				}
			
				# load and show images	
				for(i in seq_len(plot_inter_num)) {
					filename <- sprintf("%sinter_image_%03i.jpg", this@path, i)
					#print(paste("show", filename))
					inter_image <- readJPEG(filename, TRUE);
					rasterImage(inter_image, 
						xoff, 
						plot_yshift - (i + 1) * stimulus_dim[1] - i * image_dist - yoff, 
						stimulus_dim[2] + xoff, 
						plot_yshift - i * stimulus_dim[1] - i * image_dist - yoff)
				}
			}
		}
		
		ShowIntraEpisodeRecallResults <- function() 
		{
			if(plot_inter_num == 0 || plot_intra_num == 0)
				return()
		
			intra_xpos1 	<-	stimulus_dim[2] + image_dist + xoff - image_box
			intra_xpos2 	<-	(plot_intra_num + 1) * stimulus_dim[2] + plot_intra_num * image_dist + xoff + image_box + plot_pxoff
			intra_text_xpos	<-	intra_xpos1 + (intra_xpos2 - intra_xpos1) / 2
			intra_text_ypos	<-	plot_yshift - yoff - stimulus_dim[1] - image_dist + image_box + larrow_off + recall_text_off / 2
			arrow_list		<-	list(NULL, NULL, NULL, NULL)
		
			if(intra_xpos2 - intra_xpos1 >= recall_text_min_size) {
				text(
					intra_text_xpos,
					intra_text_ypos,
					"intra-episode recall", adj = c(0.5, 0.5), cex = plot_cex, col = dark_blue)
			}
		
			if(intra_xpos2 > intra_xpos1) {
				intra_arrow_ypos <- plot_yshift - yoff - stimulus_dim[1] - image_dist + image_box + larrow_off / 2
			
				arrows(
					intra_xpos1,
					intra_arrow_ypos,
					intra_xpos2,
					intra_arrow_ypos,
					col = dark_blue,
					length = larrow_length_bounded)
			}
		
			# inter-episode recall loop
			for(i in seq_len(plot_inter_num)) {
				step_intra_num	<-	min(intra_num_array[i], plot_intra_num)
				
				if(step_intra_num > 0) {
					step_plot_pxoff	<-	ifelse(step_intra_num < intra_num_array[i], plot_pxoff, 0)
					intra_xpos2 	<-	(step_intra_num + 1) * stimulus_dim[2] + step_intra_num * image_dist + xoff + image_box + step_plot_pxoff
					intra_ypos1		<-	plot_yshift - yoff - (i + 1) * stimulus_dim[1] - i * image_dist - image_box
					intra_ypos2		<-	plot_yshift - yoff - i * (stimulus_dim[1] + image_dist) + image_box
				
					rect(
						intra_xpos1,
						intra_ypos1,
						intra_xpos2,
						intra_ypos2,
						col = light_blue,
						border = dark_blue)
						
					# draw small arrows
					for(j in seq_len(ifelse(plot_intra_num < intra_num_array[i], step_intra_num + 1, step_intra_num))) {
						arrow_list[[1]] <- c(arrow_list[[1]], xoff + j * stimulus_dim[2] + (j - 1) * image_dist)
						arrow_list[[2]] <- c(arrow_list[[2]], plot_yshift - i * (stimulus_dim[1] + image_dist) - stimulus_dim[1] / 2 - yoff)
						arrow_list[[3]] <- c(arrow_list[[3]], xoff + j * (stimulus_dim[2] + image_dist))
						arrow_list[[4]] <- c(arrow_list[[4]], plot_yshift - i * (stimulus_dim[1] + image_dist) - stimulus_dim[1] / 2 - yoff)
					}
					arrows(arrow_list[[1]], arrow_list[[2]], arrow_list[[3]], arrow_list[[4]], col = black, length = sarrow_length_bounded);	
					
					# show ...
					if(plot_intra_num < intra_num_array[i]) {
						text(
							xoff + (plot_intra_num + 1) * (stimulus_dim[2] + image_dist) + points_xoff / 2,
							plot_yshift - i * (stimulus_dim[1] + image_dist) - stimulus_dim[1] / 2 - yoff,
							points_string, adj = c(0.5, 0.5), cex = plot_cex, col = black)
					}
				
					# load and show images	
					for(j in seq_len(step_intra_num)) {
						filename <- sprintf("%sintra_image_%03i_%03i.jpg", this@path, i, j)
						#print(paste("load", filename))
						intra_image <- readJPEG(filename, TRUE)
						rasterImage(intra_image, 
							j * stimulus_dim[2] + j * image_dist + xoff, 
							plot_yshift - (i + 1) * stimulus_dim[1] - i * image_dist - yoff, 
							(j + 1) * stimulus_dim[2] + j * image_dist + xoff, 
							plot_yshift - i * stimulus_dim[1] - i * image_dist - yoff)
					}
				}
			}
		}
		
		ShowStimulus <- function() 
		{
			text(xoff - image_box, plot_yshift - yoff + stimulus_text_yoff, "stimulus", adj = c(0, 1), cex = plot_cex, col = dark_green)

			rect(
				xoff - image_box,
				plot_yshift - stimulus_dim[1] - yoff - image_box,
				stimulus_dim[2] + xoff + image_box,
				plot_yshift - yoff + image_box,
				col = light_green,
				border = dark_green)
				
			rasterImage(stimulus, 
				xoff, 
				plot_yshift - stimulus_dim[1] - yoff, 
				stimulus_dim[2] + xoff, 
				plot_yshift - yoff)
		}
	
#--------------------------------------------------------------------------------------------------
	
		library(jpeg)
	
		# count available images
		inter_num		<-	0
		intra_num_array	<-	NULL
		max_intra_num	<-	0
		CountImages()
		
		# incorporate maximum values
		plot_inter_num	<-	ifelse(max_inter_steps < 0, inter_num, min(inter_num, max_inter_steps))
		plot_intra_num	<-	ifelse(max_inter_steps == 0, 0, ifelse(max_intra_steps < 0, max_intra_num, min(max_intra_num, max_intra_steps)))
		
		filename 		<-	sprintf("%sstimulus.jpg", this@path)
		stimulus 		<-	readJPEG(filename, TRUE)
		stimulus_dim	<-	dim(stimulus)
		
		# set colours
		light_blue		<-	"#f0f0ff"
		dark_blue 		<-	"#7070bf"
		light_green		<-	"#f0fff0"
		dark_green 		<-	"#70bf70"	
		light_red		<-	"#fff0f0"
		dark_red 		<-	"#bf7070"
		black			<-	"#000000"
		
		# image parameters
		image_dist 		<-	20
		image_box		<-	5
		
		# large arrow parameters
		larrow_off		<-	ifelse(inter_num > 0, 20, 0)
		larrow_length	<-	8
		
		# small arrow parameters
		sarrow_length	<-	6
		
		# text parameters
		recall_text_off			<-	ifelse(inter_num > 0, 3, 0)
		stimulus_text_yoff		<-	20
		recall_text_min_size	<-	120
		
		# ... parameters
		points_xoff		<-	30
		points_yoff		<-	20
		points_string	<-	"..."
		
		xoff			<-	image_box + ifelse(max_inter_steps == 0, 0, larrow_off + recall_text_off)
		yoff			<-	image_box + stimulus_text_yoff
		
		# plot parameters
		plot_dpi		<-	150
		plot_cex		<-	0.5
		plot_pxoff		<-	ifelse(max_inter_steps == 0, 0, ifelse(plot_intra_num > 0 && plot_intra_num < max_intra_num, image_dist + points_xoff, 0))
		plot_pyoff		<-	ifelse(plot_inter_num > 0 && plot_inter_num < inter_num, image_dist + points_yoff, 0)
		plot_xsize		<-	stimulus_dim[2] * (plot_intra_num + 1) + image_dist * plot_intra_num + image_box + xoff + plot_pxoff
		plot_ysize		<-	stimulus_dim[1] * (plot_inter_num + 1) + image_dist * plot_inter_num + image_box + yoff + plot_pyoff
		plot_yshift		<-	plot_ysize - 1
		plot_xmargin	<-	0.1
		plot_ymargin	<-	0.1
		plot_width		<-	plot_xsize / plot_dpi + 2 * plot_xmargin
		plot_height		<-	plot_ysize / plot_dpi + 2 * plot_ymargin
		
		pdf(file = "images/ETA_recall.pdf", width = plot_width, height = plot_height)
		par(bty = "n", xaxt = "n", yaxt = "n", mai = c(plot_ymargin, plot_xmargin, plot_ymargin, plot_xmargin))
		plot(NULL, NULL, xlim = c(0, plot_xsize), ylim = c(0, plot_ysize), xlab = "", ylab = "", asp = 1)
		
		larrow_length_bounded	<-	min(larrow_length / plot_dpi, larrow_off / (2 * plot_dpi))
		sarrow_length_bounded	<-	min(sarrow_length / plot_dpi, image_dist / (2 * plot_dpi))
		
		ShowStimulus()
		ShowInterEpisodeRecallResults()
		ShowIntraEpisodeRecallResults()
		
		dev.off()
	})
}
